<?php

require "config.inc.php";
require "general.inc.php";
require "ldapfunc.inc.php";

$javascript = "";

# {{{ login() returns binddn and bindpw if true, otherwise
# shows the login form 
function login() {
	# If $ldap_server && $ldap_port are set
	if ($_SESSION["ldap_server"] && $_SESSION["ldap_port"]) {
		# First connect..
		$ldap_func = new LDAPFunc($_SESSION["ldap_server"], $_SESSION["ldap_port"]) or exitOnError(ERROR_LDAP_CANT_CONNECT, $_SESSION["ldap_server"].":".$_SESSION["ldap_port"]);

		# Let's try to login, if successful skip the next stuff
		$bind = $ldap_func->bind($_SESSION["ldap_binddn"], $_SESSION["ldap_bindpw"]);
		if ($bind) {
			return $ldap_func;
		}
			echo "Bind problem: ".ldap_error($ldap_func->ldap_conn)."<BR>";
	}

	# Set defaults if nothing was isset
	if (!isset($_SESSION["ldap_server"]))
		$_SESSION["ldap_server"]	= LDAP_SERVER;
	if (!isset($_SESSION["ldap_port"]))
		$_SESSION["ldap_port"]	= LDAP_PORT;
	if (!isset($_SESSION["ldap_binddn"]))
		$_SESSION["ldap_binddn"]	= LDAP_BINDDN;
	if (!isset($_SESSION["ldap_basedn"]))
		$_SESSION["ldap_basedn"]	= LDAP_BASEDN;

	require INCLUDE_PATH."/login_form.inc";
} # }}}

# {{{ logout() - deletes the cookies (Session info..)
function logout() {
	global $javascript;
	if (session_destroy()) 
		echo "Logged out successfully, click <A HREF=\"".MAINFILE."\" TARGET=\"right\">here</A> to login again...<BR>";
	else
		echo "Error logging out!<BR>";
} # }}}

# {{{ getCachedSchema() - wraps $ldap_func->getSchema() function to add caching
# We use the neat serialize() function php has..
#
# Returns the values into $name2oid & $objectclasses
# 
function getCachedSchema($ldap_func, &$name2oid, &$objectclasses) {

	# Set the filenames of cache files
	$name2oidCacheFile = NAME2OID_CACHEFILE.".".$ldap_func->get_server();
	$objectclassesCacheFile = OBJECTCLASSES_CACHEFILE.".".$ldap_func->get_server();
	$cacheIsFine = TRUE;

	/* Can we get it from cache?
	1. If cache files exist *AND*
	2. If CACHE_EXPIRES seconds haven't passed yet since the file's mtime
	*/

	if (!file_exists($name2oidCacheFile) ||
	    !file_exists($objectclassesCacheFile))
		$cacheIsFine = FALSE; # Cache files don't exist, cache is NOT fine
	else {
		if ((filemtime($name2oidCacheFile)+CACHE_EXPIRES<time()) ||
		(filemtime($objectclassesCacheFile)+CACHE_EXPIRES<time())) {
			# Cache expired, then cache is NOT fine
			$cacheIsFine = FALSE; 
		}
	}

	
	if ($cacheIsFine) {
		if ($f = @fopen($name2oidCacheFile, "r")) {
			$str = fread($f, filesize($name2oidCacheFile)) or exitOnError(ERROR_CACHE_CANT_READ, $name2oidCacheFile);
			$name2oid = unserialize($str);
			fclose($f);
		}
		else
			exitOnError(ERROR_CACHE_CANT_READ, $name2oidCacheFile);

		if ($f = @fopen($objectclassesCacheFile, "r")) {
			$str = fread($f, filesize($objectclassesCacheFile)) or exitOnError(ERROR_CACHE_CANT_READ, $objectclassesCacheFile);
			$objectclasses = unserialize($str);
			fclose($f);
		}
		else
			exitOnError(ERROR_CACHE_CANT_READ, $objectclassesCacheFile);
	}
	else { # Cache is no good
		/* Read the schema from LDAP */
		$ldap_func->getSchemaHash($name2oid, $objectclasses);

		/* Save it in cache */
		umask(077); # We don't want the schema world readable..
		if ($f = @fopen($name2oidCacheFile, "w")) {
			fwrite($f, serialize($name2oid)) or exitOnError(ERROR_CACHE_CANT_WRITE, $name2oidCacheFile);
			fclose($f);
		}
		else
			exitOnError(ERROR_CACHE_CANT_WRITE, $name2oidCacheFile);

		if ($f = @fopen($objectclassesCacheFile, "w")) {
			fwrite($f, serialize($objectclasses)) or exitOnError(ERROR_CACHE_CANT_WRITE, $objectclassesCacheFile);
			fclose($f);
		}
		else
			exitOnError(ERROR_CACHE_CANT_WRITE, $objectclassesCacheFile);
	}
}
# }}}

# {{{ viewEntry() displays the given entry's attributes and values,
# according to the schema of its objectclasses
# If no entry is given we display an empty entry of the specified object claases
#
# For each attribute we'll show $empties values in addition
function viewEntry($ldap_func, $entry, $empties = 0, $dn = "", $objectclasses = array()) {
	global $attr_desc; # FIXME Global is ugly

	$info = array();

	if (!$ldap_func || !$entry && !count($objectclasses)) exitOnError(ERROR_FEW_ARGUMENTS);

	# Will contain a list of attributes later
	$may = $must = array();

	# Get the schema contents
#	$ldap_func->getSchemaHash($name2oid, $schema_objectclasses);
	getCachedSchema($ldap_func, $name2oid, $schema_objectclasses);



	# In the next lines we'll fill this array with the list of objectclasses
	$objectclasses_list = array();

	if ($entry) {
		# If we are viewing an existing entry, read objectclass-list from the directory
		
		$sr = ldap_read($ldap_func->ldap_conn, $entry, "(objectclass=*)") or exitOnError(ERROR_LDAP_CANT_SEARCH);
		$info = ldap_get_entries($ldap_func->ldap_conn, $sr) or exitOnError(ERROR_LDAP_CANT_SEARCH);

		# Fill objectclasses_list with the objectclasses of that attribute
		for ($i = 0; $i < $info[0]["objectclass"]["count"]; $i++) {
			array_push($objectclasses_list, $info[0]["objectclass"][$i]);
		}
	}
	else {
		# If it's an empty entry, objectclasses are given arguments
		$objectclasses_list = $objectclasses;

		# Add the values of the objectclass attribute in a patchy-way
		$info[0]["objectclass"] = array();
		foreach ($objectclasses_list as $objectclass) {
			array_push($info[0]["objectclass"], $objectclass);
		}
		$info[0]["objectclass"]["count"] = count($objectclasses_list);
	}


	# Now when we have $objectclass_list, get the attrs of each objectclass
	# And merge them (an entry might have several objectclasses)
	foreach ($objectclasses_list as $objectclass) {
#		echo "Reading attributes of objectclass ".$objectclass."<BR>\n";
#		$objectclass = strtolower($info[0]["objectclass"][$i]);

		# Recursively get the may/must attributes of $objectclass and it's
		# superiors.
		$maymust = $ldap_func->getMayMust($objectclass, $schema_objectclasses, $name2oid);

		$must = array_merge($must, $maymust["must"]);
		$may = array_merge($may, $maymust["may"]);
	}

#echo $info[0]["objectclass"][0];

	# Remove dups + sort
	$must = array_unique($must);
	$may  = array_unique($may);
	asort($must);
	asort($may);

	# ENTRY TITLE COMES HERE
	if ($entry) echo "<CENTER><H4><B>".$entry."</B></H4></CENTER><BR>\n";
	echo "One <A HREF=\"".MAINFILE."?do=view_entry&entry=".urlencode($entry)."&empties=".($empties+1)."\">more</A> / <A HREF=\"".MAINFILE."?do=view_entry&entry=".urlencode($entry)."&empties=".($empties-1)."\">less</A> empty value field (for each attribute)<BR>";



	echo "<FORM NAME=\"form\" METHOD=\"post\" ACTION=\"".MAINFILE."\"";
	if (ENABLE_JAVASCRIPT) echo " onSubmit=\"javascript:return confirm('Are you sure you want to commit a \''+this.chosen_action.value+'\' operation on this entry?');\"";
	echo ">\n";
	if ($entry )echo "<INPUT TYPE=\"hidden\" NAME=\"entry\" VALUE=\"".$entry."\">\n";
	echo "<INPUT TYPE=\"hidden\" NAME=\"chosen_action\" VALUE=\"none\">\n";
	echo "<TABLE BORDER=\"0\" CELLSPACING=\"3\" CELLPADDING=\"1\">\n";

	# Show the dn before anything else
	if ($info[0]["dn"]) $dn = $info[0]["dn"];
	echo "<TR BGCOLOR=\"#BBBBCC\"><TD WIDTH=\"27%\"><FONT SIZE=\"+1\"><B><ACRONYM TITLE=\"Distinguished Name\">dn</ACRONYM>:&nbsp;<FONT SIZE=\"-2\">[&nbsp;<A HREF=\"".MAINFILE."?do=modrdn_form&entry=".urlencode($dn)."\">modify dn</A>&nbsp;]&nbsp;</B></FONT></TD><TD><INPUT TYPE=\"text\" NAME=\"dn\" VALUE=\"".$dn."\" SIZE=\"".INPUT_TEXT_SIZE."\"></TD></TR>\n";

	$status = MUST;
	$maymust = &$must;
	while ( 1 ) {
		foreach ($maymust as $attr) {

			# See if there's a description to this specific attribute
			if ($attr_desc[$attr]) {
				$acronym_begin = "<ACRONYM TITLE=\"".$attr_desc[$attr]."\">";
				$acronym_end = "</ACRONYM>";
			} else {
				$acronym_begin = "";
				$acronym_end = "";
			}

			# Mark the attribute if it's a MUST
			if ($status == MUST) {
				$mark_begin = "<B>";
				$mark_end   = "</B>";
			}
			else {
				$mark_begin = "";
				$mark_end   = "";
			}

			$val  = $info[0][strtolower($attr)];

			# Show all the existing values (if none, at least
			# one empty!) + $empty_values empties in addition
			for ($j = 0; ($j < ( max($val["count"], 1) + $empties)); $j++) {
				# Very stupid color changing
				if ($bgcolor == COLOR_BG2) $bgcolor = COLOR_BG1;
				else
					$bgcolor = COLOR_BG2;
		
				if ($j > $val["count"]) $value = "";
				else
					$value = $val[$j];
				echo "<TR BGCOLOR=\"".$bgcolor."\"><TD WIDTH=\"25%\"><FONT COLOR=\"".COLOR_FGTEXT."\">".$mark_begin.$acronym_begin.$attr.$acronym_end.$mark_end."</FONT>:</TD><TD><INPUT TYPE=\"text\" NAME=\"".$attr."[]\" VALUE=\"".$value."\" SIZE=\"".INPUT_TEXT_SIZE."\"></TD></TR>\n";

			}

		}

		if ($status == MAY) break;
		if ($status == MUST) { $status = MAY; $maymust = &$may; }

	}
	echo "</TABLE><BR>\n<CENTER><TABLE BORDER=\"0\" CELLSPACING=\"2\" CELLPADDING=\"2\" WIDTH=\"75%\"><TR ALIGN=\"center\">";
	if ($entry) {
		echo "<TD><INPUT TYPE=\"submit\" NAME=\"submit\" VALUE=\"Modify\"";
		if (ENABLE_JAVASCRIPT) echo " onClick=\"javascript:this.form.chosen_action.value='Modify';\"";
		echo "></TD>";
	}
	echo "<TD><INPUT TYPE=\"submit\" NAME=\"submit\" VALUE=\"New\"";
	if (ENABLE_JAVASCRIPT) echo " onClick=\"javascript:this.form.chosen_action.value='New';\"";
	echo "></TD>";
	if ($entry) {
		echo "<TD><INPUT TYPE=\"submit\" NAME=\"submit\" VALUE=\"Delete\"";
		if (ENABLE_JAVASCRIPT) echo " onClick=\"javascript:this.form.chosen_action.value='Delete';\"";
		echo "></TD>";
	}
	echo "</TR></TABLE></CENTER>\n";
	echo "</FORM>\n";
	

}
# }}}

# {{{ modifyEntry() gets the modifications and decides what to do...
function modifyEntry($ldap_func, $post_vars) {
	#$add_hash = array();
	#$del_hash = array();
	#$replace_hash = array();

	if (!$ldap_func || !is_array($post_vars)) exitOnError(ERROR_FEW_ARGUMENTS);
	$dn = $post_vars["entry"];

	$sr = ldap_read($ldap_func->ldap_conn, $dn, "(objectClass=*)") or exitOnError(ERROR_LDAP_CANT_SEARCH);
	$entry = ldap_first_entry($ldap_func->ldap_conn, $sr) or exitOnError(ERROR_LDAP_CANT_SEARCH);
	$attributes = ldap_get_attributes($ldap_func->ldap_conn, $entry);


	#
	# Create add_hash
	# 

	# Pass on each posted attribute
	while ( list($attr, $posted_values) = each($post_vars) ) {
		# We care about this var only if it's an array
		if (!is_array($posted_values)) continue;

		# If $attr wasn't found in ldap (means that it has no value in ldap yet) - add all values to the 'add_hash'
		if (!isset($attributes[$attr])) {
			foreach ($posted_values as $posted_value) {
				if (!$posted_value) continue; # Skip Empty
				if (!is_array($add_hash[$attr])) $add_hash[$attr] = array();
				array_push($add_hash[$attr], $posted_value);
			}
			continue;
		}

		$ldap_values = ldap_get_values($ldap_func->ldap_conn, $entry, $attr);

		foreach ($posted_values as $posted_value) {
			if (!$posted_value) continue; # Ignore empty values
			if (!in_array($posted_value, $ldap_values)) {
				if (!is_array($add_hash[$attr])) $add_hash[$attr] = array();
				array_push($add_hash[$attr], $posted_value);
			}
		}
	}


	#
	# Create del_hash
	#

	# Now pass on each attribute from ldap and see if it has a real
	# value in $posted_vars
	for ($i = 0; $i < $attributes["count"]; $i++) {
		$attr = $attributes[$i];
		$ldap_values = ldap_get_values($ldap_func->ldap_conn, $entry, $attr);

		for ($j = 0; $j < $ldap_values["count"]; $j++) {
			if (!in_array($ldap_values[$j], $post_vars[$attr]))  {
				if (!is_array($del_hash[$attr])) $del_hash[$attr] = array();
				array_push($del_hash[$attr], $ldap_values[$j]);
			}
		}
	}

	#
	# Create replace_hash
	#
	
	# Now we have two hashes, add_hash and del_hash. If we both del a value of attribute 'x' and add another value - we'd rather REPLACE this value instead of deleting and adding (as long as attribute 'x' has only a single value!).. (MUST values cannot be deleted, that is)
	if (is_array($del_hash))
	while ( list($attr, $values) = each($del_hash) ) {

		# If it has more than one value in the directory, we cannot
		# replace it.
		if ($attributes[$attr]["count"] > 1) continue;

		# does the same attribute from del_hash exist in add_hash too?
		if (is_array($add_hash[$attr]) && count($add_hash[$attr])) {
			$add = array_shift($add_hash[$attr]);
			$del = array_shift($del_hash[$attr]);

			if (!is_array($replace_hash[$attr])) $replace_hash[$attr] = array();
			array_push($replace_hash[$attr], $add);
		}
	}

	$operations = array("del", "add", "replace");

	foreach ($operations as $op) {
		$varname = $op."_hash"; #either del/add/replace_hash
		if (!is_array(${$varname})) continue; # If empty, skip
		echo "<B>".$op.":</B><BR>";

		reset(${$varname});
		while (list($attr, $values) = each(${$varname})) {
			foreach($values as $value) {
				$entry = array();

				echo $attr.": ".$value." - ";
				$entry[$attr] = $value;

				switch($op) {
					case "del": $result = @ldap_mod_del($ldap_func->ldap_conn, $dn, $entry); break;
					case "add": $result = @ldap_mod_add($ldap_func->ldap_conn, $dn, $entry); break;
					case "replace": $result = @ldap_mod_replace($ldap_func->ldap_conn, $dn, $entry); break;
				}
				if ($result) echo "OK";
				else
					echo "FAILED: ".ldap_error($ldap_func->ldap_conn);
				echo "<BR>";
			}
		}

	}

} # }}}

# {{{ deleteEntry() deletes the given entry from the directory
function deleteEntry($ldap_func, $dn) {

	echo "<B>Deleting</B>... ";

	$result = ldap_delete($ldap_func->ldap_conn, $dn);

	if ($result) echo "OK";
	else
		echo "FAILED: ".ldap_error($ldap_func->ldap_conn);
	echo "<BR>";
}
# }}}

# {{{ newEntry() creates a new LDAP entry according to the given parameters
function newEntry($ldap_func, $post_vars) {
	$dn = $post_vars["dn"];
	echo "dn: ".$dn."<BR>";
	# Let's construct $entry, which will contain the future attrs/vals
	while (list($attr, $values) = each($post_vars)) {

		# Skip if the value is not an array- it means that it's not
		# a variable ment to be an attribute
		if (!is_array($values)) continue;

		foreach ($values as $value) {
			# Skip if value is empty
			if (!$value) continue;

			if (!is_array($entry[$attr])) $entry[$attr] = array();
			array_push($entry[$attr], $value);
			echo $attr.": ".$value."<BR>";
		}
	}

	echo "<BR><B>Adding...</B> ";
	$result = @ldap_add($ldap_func->ldap_conn, $dn, $entry);
	if ($result) echo "OK";
	else
		echo "Failed: ".ldap_error($ldap_func->ldap_conn);

} # }}}

# {{{ search_form() displays a search form
function search_form($ldap_basedn) {
	include INCLUDE_PATH."/search_form.inc";
} # }}}

# {{{ search() search and return the results as an array
function search($ldap_func, $post_vars) {

	$binddn = $post_vars["basedn"];
	$filter = $post_vars["filter"];
	$scope  = $post_vars["scope"];

	if (!$binddn || !$filter || !$scope)
		exitOnError(ERROR_FEW_ARGUMENTS);

	$info = $ldap_func->search($binddn, $filter, $scope);

	echo "<H2><CENTER>".$info["count"]." result(s)</CENTER></H2><BR>\n";
	if (!$info["count"]) return;

	for ($i = 0; $i < $info["count"]; $i++) {
		$dn = $info[$i]["dn"];
		echo "<A HREF=\"".MAINFILE."?do=view_entry&entry=".urlencode($dn)."\">".$dn."</A><BR>\n";
	}
} # }}}

# {{{ choose_entrytype() displays a form for choosing a new entry type
function choose_entrytype($entry_types, $ldap_func, $parent) {

	$ldap_func->getSchemaHash($name2oid, $schema_objectclasses);
	
	require INCLUDE_PATH."/choose_entrytype_form.inc";
} # }}}

# {{{ new_form() displays a form for adding a new entry
function new_form($ldap_func, $entry_types, $post_vars) {
	$entry_type = $post_vars["entry_type"] or exitOnError(ERROR_FEW_ARGUMENTS);

	$objectclasses_list = array();

	if ($post_vars["dn"]) $dn = ",".$post_vars["dn"];
	else
		$dn = "";

	# If custom, list of objectclasses is given as an argument
	if ($entry_type == "custom") {
		if (!count($post_vars["custom_objectclasses"])) exitOnError(ERROR_FEW_ARGUMENTS);
		$objectclasses_list = $post_vars["custom_objectclasses"];
	}
	else # If a specific entry type was chosen, list is in $entry_types
		$objectclasses_list = $entry_types[$entry_type];

	viewEntry($ldap_func, "", 0, $dn, $objectclasses_list);

	
} # }}}

# {{{ modrdn_form() displays a form for modifying the dn
function modrdn_form($ldap_func, $entry) {
	if (eregi("^([^,]+),(.*)$", $entry, $regs)) {
		$rdn		= $regs[1];
		$superior	= $regs[2];
	}
	include INCLUDE_PATH."/modrdn_form.inc";

} # }}}

# {{{ modrdn() - Modify the RDN and/or the Parent ( = rename )
function modrdn($ldap_func, $post_vars) {
	$entry		= $post_vars["entry"];
	$newrdn		= $post_vars["newrdn"];
	$deleteoldrdn	= $post_vars["deleteoldrdn"];
	$newsuperior	= $post_vars["newsuperior"];

	echo "dn: ".$entry."<BR>\n";
	echo "newrdn: ".$newrdn."<BR>\n";
	echo "deleteoldrdn: ".$deleteoldrdn."<BR>\n";
	if ($newsuperior) echo "newsuperior: ".$newsuperior."<BR>\n";

	$result = ldap_rename($ldap_func->ldap_conn, $entry, $newrdn, $newsuperior, $deleteoldrdn);
	if ($result) echo "OK";
	else
		echo "Failed: ".ldap_error($ldap_func->ldap_conn);
	echo "<BR>\n";
} # }}}


# Here we BEGIN
session_start();
$submit		= $HTTP_POST_VARS["submit"];
$do		= $HTTP_GET_VARS["do"];
if (isset($HTTP_POST_VARS["ldap_server"]))
	$_SESSION["ldap_server"] = $HTTP_POST_VARS["ldap_server"];
if (isset($HTTP_POST_VARS["ldap_port"]))
	$_SESSION["ldap_port"] = $HTTP_POST_VARS["ldap_port"];
if (isset($HTTP_POST_VARS["ldap_basedn"]))
	$_SESSION["ldap_basedn"] = $HTTP_POST_VARS["ldap_basedn"];
if (isset($HTTP_POST_VARS["ldap_binddn"]))
	$_SESSION["ldap_binddn"] = $HTTP_POST_VARS["ldap_binddn"];
if (isset($HTTP_POST_VARS["ldap_bindpw"]))
	$_SESSION["ldap_bindpw"] = $HTTP_POST_VARS["ldap_bindpw"];


if ($submit == "Login" || $submit == "Anonymous Login" || $submit == "Delete" || $submit == "New" || $submit == "Modify" || $submit == "Modrdn" || $do == "logout") {
	$javascript .= "top.left.location.reload();\n"; # Refresh after login
}

require INCLUDE_PATH."/header.inc";

# If anonymous login, act as there is no binddn nor bindpw
if ($submit == "Anonymous Login") {
	$ldap_binddn = ""; $ldap_bindpw = "";
}
$ldap_func = login();

# Sanity checks on parameters
$empties = $HTTP_GET_VARS["empties"]; if ($empties < 0) $empties = 0; elseif ($empties > 5) $empties = 5;
echo $empties;

if ($do) {
	switch ($do) {
		case "logout": logout(); break;
		case "search_form": search_form(LDAP_BASEDN); break;
		case "modrdn_form": modrdn_form($ldap_func, $HTTP_GET_VARS["entry"]); break;
		case "view_entry": viewEntry($ldap_func, $HTTP_GET_VARS["entry"], $empties); break;
		case "choose_entrytype": choose_entrytype($entry_types, $ldap_func, $HTTP_GET_VARS["parent"]); break;
		default: exitOnError(ERROR_BAD_OP, $do);
	}
}

if ($submit) { # If it's a form which was submitted (modify/del/add...)
	switch ($submit) {
		case "Modrdn": modrdn($ldap_func, $HTTP_POST_VARS); break;
		case "Modify": modifyEntry($ldap_func, $HTTP_POST_VARS); break;
		case "Delete": deleteEntry($ldap_func, $HTTP_POST_VARS["entry"]); break;
		case "New": newEntry($ldap_func, $HTTP_POST_VARS); break;
		case "Search": search($ldap_func, $HTTP_POST_VARS); break;
		case "Create": new_form($ldap_func, $entry_types, $HTTP_POST_VARS);
		case "Anonymous Login":
		case "Login": break;
		default: exitOnError(ERROR_BAD_OP, $submit);
	}
}


require INCLUDE_PATH."/footer.inc";


